#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 3/6/22

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import os, string, sys,time
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def create_an_easter_egg(eggColor, usePattern, pattern):
		
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    
    
    #Create a background
    width = 600
    height = 800
    img = gimp.Image(width, height, RGB)
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_context_set_background((284, 282, 187))
    pdb.gimp_drawable_fill(backLayer, FILL_BACKGROUND)
    pdb.plug_in_hsv_noise(img, backLayer, 8, 3, 10, 100)

    #egg
    eggW = 400
    eggH = 500
    eggLayer = pdb.gimp_layer_new(img, eggW, eggH, RGBA_IMAGE, "Easter Egg", 100, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_image_add_layer(img, eggLayer, 0)
    pdb.gimp_context_set_foreground(eggColor)
    pdb.gimp_context_set_background((20, 20,20))
    pdb.gimp_drawable_fill(eggLayer, FILL_FOREGROUND)
    
    #factor = min (float(img.width) / eggLayer.width, float(img.height) / eggLayer.height)
    eggLayer.set_offsets((img.width - eggLayer.width) / 2, (img.height - eggLayer.height) / 2)
    
    #Create oval shape
    pdb.plug_in_applylens(img, eggLayer, 
                                      3,  #refraction
                                  FALSE,  #keep_surroundings
                                  FALSE,  #set_background
                                  FALSE)  #set_transparent
    #Resize the layer
    pdb.gimp_layer_resize_to_image_size(eggLayer)
    
    #Make the egg shape
    pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 10,1,100,50,16,-90,0")
    pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 22,4,14,50,32,-90,0")
    pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 22,4,14,50,32,-90,0")
    pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 14,2,100,50,26,-90,0")

    #Paint the egg
    pdb.gimp_image_select_item(img, 2, eggLayer)
    pdb.plug_in_sel2path(img, eggLayer)
    pdb.gimp_selection_none(img)
    vectors = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))
    pdb.gimp_vectors_to_selection(vectors, 
                                        2,  #operation - channel_op_replace
                                     TRUE,  #antialias
                                    FALSE,  #feather
                                        0,  #feather_radius_x
                                        0)  #feather_radius_y
    
    #Create a copy for inner shadow
    innShadow = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Inner Shadow", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, innShadow, 0)
    pdb.gimp_drawable_fill(innShadow, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground((20, 20,20))
    pdb.gimp_edit_blend(innShadow, 
                                2,   #blend_mode - fg to transparent
                                0,   #paint_mode - normal
                                2,   #gradient_type - radial
                              100,   #opacity, 
                                0,   #offset
                                0,   #repeat
                             TRUE,   #reverse
                            FALSE,   #supersample
                                0,   #max_depth
                                0,   #threshold
                             TRUE,   #dither
                              300,   #x1
                              250,   #y1
                              300,   #x2
                              800)   #y2
    
    pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-fx_distort_lens 0.55,0,0.4,50,50,0")
    pdb.gimp_selection_none(img)
    
    #Drop shadow
    dropShadow = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Drop Shadow", 60, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, dropShadow, 2)
    pdb.gimp_drawable_fill(dropShadow, FILL_TRANSPARENT)
    pdb.gimp_context_set_background((20, 20,20))
    pdb.gimp_image_select_item(img, 2, eggLayer)
    pdb.gimp_edit_bucket_fill(dropShadow, BUCKET_FILL_BG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    pdb.plug_in_gauss(img, dropShadow, 30, 30, 1)   #method=1
    
    #Light reflection
    highlight = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Highlight", 20, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, highlight, 0)
    pdb.gimp_drawable_fill(highlight, FILL_TRANSPARENT)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_context_set_brush("2. Hardness 050")
    pdb.gimp_context_set_brush_size(100)
    pdb.gimp_context_set_brush_aspect_ratio(0.20)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(10)
    pdb.gimp_context_set_brush_hardness(0.5)
    pdb.gimp_context_set_brush_force(0.5)
    pdb.gimp_paintbrush_default(highlight, 2, [300, 270])
    
    
    
    #Make active the bottom layer and done
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, backLayer)
        
    if usePattern:     
        #Create a background
        width = 600
        height = 800
        img = gimp.Image(width, height, RGB)
        backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, backLayer, 0)
        pdb.gimp_context_set_background((284, 282, 187))
        pdb.gimp_drawable_fill(backLayer, FILL_BACKGROUND)
        pdb.plug_in_hsv_noise(img, backLayer, 8, 3, 10, 100)
        
        #egg
        eggW = 400
        eggH = 500
        eggLayer = pdb.gimp_layer_new(img, eggW, eggH, RGBA_IMAGE, "Easter Egg", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, eggLayer, 0)
        pdb.gimp_context_set_foreground(eggColor)
        pdb.gimp_context_set_background((20, 20,20))
        pdb.gimp_drawable_fill(eggLayer, FILL_FOREGROUND)
        
        #factor = min (float(img.width) / eggLayer.width, float(img.height) / eggLayer.height)
        eggLayer.set_offsets((img.width - eggLayer.width) / 2, (img.height - eggLayer.height) / 2)
        
        #Create oval shape
        pdb.plug_in_applylens(img, eggLayer, 
                                          3,  #refraction
                                      FALSE,  #keep_surroundings
                                      FALSE,  #set_background
                                      FALSE)  #set_transparent
        #Resize the layer
        pdb.gimp_layer_resize_to_image_size(eggLayer)
        
        #Make the egg shape
        pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 10,1,100,50,16,-90,0")
        pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 22,4,14,50,32,-90,0")
        pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 22,4,14,50,32,-90,0")
        pdb.plug_in_gmic_qt(img, eggLayer, 1, 0, "-gui_rep_pw 14,2,100,50,26,-90,0")
        
        #Apply pattern
        pdb.gimp_image_select_item(img, 2, eggLayer)
        pdb.plug_in_sel2path(img, eggLayer)
        pdb.gimp_selection_none(img)
        vectors = pdb.gimp_image_get_vectors_by_name(img, _("Selection"))
        pdb.gimp_vectors_to_selection(vectors, 
                                            2,  #operation - channel_op_replace
                                         TRUE,  #antialias
                                        FALSE,  #feather
                                            0,  #feather_radius_x
                                            0)  #feather_radius_y
               
        patLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Pattern", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, patLayer, 0)
        pdb.gimp_drawable_fill(patLayer, FILL_TRANSPARENT)
        pdb.gimp_context_set_pattern(pattern)
        pdb.gimp_edit_bucket_fill(patLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
        pdb.plug_in_gmic_qt(img, patLayer, 1, 0, "-fx_distort_lens 0.55,0,0.4,50,50,0")
        pdb.plug_in_gmic_qt(img, patLayer, 1, 0, "-fx_distort_lens 0.55,0,0.4,50,50,0")
        
        #Create a copy for inner shadow
        innShadow = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Inner Shadow", 80, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, innShadow, 0)
        pdb.gimp_drawable_fill(innShadow, FILL_TRANSPARENT)
        pdb.gimp_context_set_foreground((20, 20,20))
        pdb.gimp_edit_blend(innShadow, 
                                    2,   #blend_mode - fg to transparent
                                    0,   #paint_mode - normal
                                    2,   #gradient_type - radial
                                  100,   #opacity, 
                                    0,   #offset
                                    0,   #repeat
                                 TRUE,   #reverse
                                FALSE,   #supersample
                                    0,   #max_depth
                                    0,   #threshold
                                 TRUE,   #dither
                                  300,   #x1
                                  250,   #y1
                                  300,   #x2
                                  800)   #y2
        
        pdb.plug_in_gmic_qt(img, innShadow, 1, 0, "-fx_distort_lens 0.55,0,0.4,50,50,0")
        pdb.gimp_selection_none(img)
        
        #Drop shadow
        dropShadow = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Drop Shadow", 60, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, dropShadow, 3)
        pdb.gimp_drawable_fill(dropShadow, FILL_TRANSPARENT)
        pdb.gimp_context_set_background((20, 20,20))
        pdb.gimp_image_select_item(img, 2, eggLayer)
        pdb.gimp_edit_bucket_fill(dropShadow, BUCKET_FILL_BG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
        pdb.gimp_selection_none(img)
        pdb.plug_in_gauss(img, dropShadow, 30, 30, 1)   #method=1
        
        #Light reflection
        highlight = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Highlight", 30, LAYER_MODE_NORMAL)
        pdb.gimp_image_add_layer(img, highlight, 0)
        pdb.gimp_drawable_fill(highlight, FILL_TRANSPARENT)
        pdb.gimp_context_set_foreground("White")
        pdb.gimp_context_set_brush("2. Hardness 075")
        pdb.gimp_context_set_brush_size(270)
        pdb.gimp_context_set_brush_aspect_ratio(-0.60)
        pdb.gimp_context_set_brush_angle(0)
        pdb.gimp_context_set_brush_spacing(10)
        pdb.gimp_context_set_brush_hardness(0.5)
        pdb.gimp_context_set_brush_force(0.5)
        pdb.gimp_paintbrush_default(highlight, 2, [300, 270])
        
        #Make active the bottom layer and done
        backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
        pdb.gimp_image_set_active_layer(img, backLayer)
       
    #Process
    pdb.gimp_display_new(img) 
    
    
    #Set the defaults
    pdb.gimp_context_pop()	
   
register(
    "create_an_easter_egg",
    "Creates an easter egg for Easter",
    "Creates an easter egg for Easter",
    "Pocholo M",
    "Pocholo M",
    "2022",
    "Easter Egg",
    "",
    [    
    (PF_COLOR,    "eggColor", "Egg Color:", (0, 255, 255)),
    (PF_OPTION,     "usePattern", "Choose Pattern?", 0, ["Don't use", "Use"]),
    (PF_PATTERN,  "pattern", "Pattern:", "Easter egg pattern #3.png", 0),
    ],
    [],
    create_an_easter_egg, menu="<Image>/File/Create/Create an Easter Egg",
    domain=("gimp20", gimp.locale_directory))
main()				       